/*
 * budg.h
 *
 * Budget related definitions
 *
 * Ville Virrankoski, 1995
 */

#define civ_taxes(x,y)        (0.5 + (x * y/100 * etu * money_civ))
#define uw_taxes(x,y)         (0.5 + (x * y/100 * etu * money_uw))

#define mil_payment(x)        (x * etu * money_mil)

void fill_update_array();
int bud_key();
int *get_wp();
int gt_bg_nmbr();
void pt_bg_nmbr();
void get_materials();

extern int mil_dbl_pay;
