/*
 * version.h
 *
 * describes major, minor, and release level
 *
 * $Revision: 2.31 $ $Date: 1995/10/25 20:21:01 $
 */

#ifndef _VERSION_H_
#define _VERSION_H_

#define EMP_VERS_MAJOR	3
#define EMP_VERS_MINOR	0
#define EMP_VERS_PATCH	12

#define	KSU_DIST	1.04
#define	CHAINSAW_DIST	4.00

#endif /* _VERSION_H_ */
